%eyam
% GUI for Eyam S,I,R epidemic model.
% S = Susceptible, I = Infective, R = Removed (i.e. Recovered + Dead)
% LAST UPDATED by Andy French Mar 2020

function varargout = eyam(varargin)

%% Initialization code - DO NOT EDIT
gui_Singleton = 1;
gui_State = struct('gui_Name',       mfilename, ...
    'gui_Singleton',  gui_Singleton, ...
    'gui_OpeningFcn', @eyam_OpeningFcn, ...
    'gui_OutputFcn',  @eyam_OutputFcn, ...
    'gui_LayoutFcn',  [] , ...
    'gui_Callback',   []);
if nargin && ischar(varargin{1})
    gui_State.gui_Callback = str2func(varargin{1});
end

if nargout
    [varargout{1:nargout}] = gui_mainfcn(gui_State, varargin{:});
else
    gui_mainfcn(gui_State, varargin{:});
end

%%

%% EYAM EQUATIONS SOLVER %%

%%  Semi-analytic method, based upon tmax, Imax, alpha, eta
function [S,I,R,t,rho,N,R0,SSD] = eyam_model( tmax, Imax, a, eta, Ni,...
    tdata, Idata, Sdata, Rdata, use_S_and_R )

%Compute D/rho limits
zplus = -log( 1 - eta ) - log( -log( 1-eta )/eta );
zminus = -log( -log(1-eta)/eta );

%Compute xmax = Imax/rho
xmax = -log( 1 - eta )/eta - 1 - log( -log( 1-eta )/eta );

%Compute rho
rho = Imax/xmax;

%Compute total population size N and Basic Reproduction Number R0
N = round( Imax + rho - rho*zminus );
R0 = N/rho;

%Determine z and tor
[z,tor] = tor_of_z( zminus,zplus, Ni, xmax );
[z,tor] = tor_of_z_cspline( zminus,zplus, Ni, xmax );

%Determine t,R,S,I
t = tor/a + tmax;
R = rho*( z - zminus );
x = xmax + 1 - exp(-z) - z;
y = exp(-z);
S = rho*y;
I = rho*x;

%Determine Sum of Squared Differences between I and Idata etc
II = interp1( t, I, tdata,'spline','extrap' );
SS = interp1( t, S, tdata,'spline','extrap'  );
RR = interp1( t, R, tdata,'spline','extrap'  );
SSD = sum( (Idata-II).^2 );
if use_S_and_R==1
    SSD = SSD + sum( (Sdata-SS).^2 ) + sum( (Rdata-RR).^2 ) ;
end

%%

%% Kermack and McKendrick (K&K) approximate solution of the Eyam equations.
function [S,I,R,N,R0] = K_and_K_eyam_model( t, rho, Imax, tmax, a )
xmax = Imax/rho;
tau = a*(t-tmax);
x = xmax*( sech( sqrt( 0.5*xmax )*tau ).^2 );
z = sqrt(2*xmax)*tanh( sqrt( 0.5*xmax )*tau );
y = exp(-z);
I = rho*x;
S = rho*y;
R = rho*(z+sqrt( 2*xmax ));
N = Imax + rho + rho*sqrt( 2*xmax );
R0 = N/rho;

%%

%% Euler solver
function [S,I,R,t] = euler( S0,I0, a, b, dt, tend )

%Initialize output vectors for t,I,S,R
t = 0 : dt : tend;
N = length(t);
S = S0*ones(1,N);
I = I0*ones(1,N);
R = zeros(1,N);

%Loop through vectors to compute t, I, S, R.
%using the Euler first order differential equation method
for n=2:N
    t(n) = t(n-1) + dt;
    I(n) = I(n-1) + dt*( b*S(n-1)*I(n-1) - a*I(n-1) );
    S(n) = S(n-1) - dt*b*S(n-1)*I(n-1);
    R(n) = R(n-1) + dt*a*I(n-1);
end

%%

%% Eyam opening function. This executes just before eyam is made visible.
function eyam_OpeningFcn(hObject, eventdata, handles, varargin)

% Choose default command line output for eyam
handles.output = hObject;

%Turn off warnings (e.g. 'divide by zero')
warning off

%Set up global variable d which can be accessed by all functions
global d
d.h = handles;

%Center figure
figcentre( d.h.FIGUREeyam );

%Set defaults
eyam_defaults

%Update eyam
eyam_update;

%%

% Default parameters
function eyam_defaults
global d

% Model parameters (Mompesson model)
d.Imax = 26.76;
d.tmax = 0.900;
d.eta = 0.6851;
d.a = 2.987;

% Number of integration steps for tor(z) evaluation
d.Ni = 1000;

% Timestep /months for Euler solver
d.dt = 0.01;

%Number of plot points for Euler solver to show
d.N_euler = 20;

% Edit box max/min values etc
d.Imaxmax = 35;
d.Imaxmin = 25;
d.tmaxmax = 1.5;
d.tmaxmin = 0.5;
d.amax = 4;
d.amin = 2;
d.etamax = 0.99;
d.etamin = 0.01;

% Define time t scale values /months
d.tscalemax = 4;
d.tscalemin = -1;

% Define vertical axis scale
d.ISDscalemax = 250;
d.ISDscalemin = 0;

% Fontsize and markersize for plots
d.fsize = 12;
d.msize = 14;
d.plot_and_save_PNG = 0;

% Default data overlay (Mompesson)
d.t_mompesson = [0,0.51,1.02,1.53,2.04,2.55,3.57];
d.I_mompesson = [14.5,22,29,21,8,8,0];
d.S_mompesson = [235,201,153.5,121,108,97,83];
d.R_mompesson = d.I_mompesson(1)+ d.S_mompesson(1) - d.I_mompesson - d.S_mompesson;
d.tdata = d.t_mompesson;
d.Idata = d.I_mompesson;
d.Sdata = d.S_mompesson;
d.Rdata = d.R_mompesson;
d.Ddata = d.Rdata; %Assume everyone died from the plague
d.overlay_data = 1;
d.default_model = 1;

% S and R as well as I?
d.overlay_SIR = 1;

%Fix axis scale
d.fix_axis_scale = 1;

%Plot 3D SSD surface during optimization?
d.plot_3D_SSD = 0;

%Use S and R data in SSD calculation
d.use_S_and_R = 0;

%Fix R=0 at t=0
d.fix_R0 = 1;

%Plot Euler solution?
d.plot_euler = 0;

%Set GUI defaults
update_gui_items

%%

%Update GUI items
function update_gui_items
global d

%a
set( d.h.EDITa,'string', num2str(d.a,4) );
set( d.h.EDITamax,'string', num2str(d.amax,4) );
set( d.h.EDITamin,'string', num2str(d.amin,4) );
set( d.h.SLIDERa,'value',(d.a - d.amin)/(d.amax - d.amin) );

%tmax
set( d.h.EDITtmax,'string', num2str(d.tmax,3) );
set( d.h.EDITtmaxmax,'string', num2str(d.tmaxmax,3) );
set( d.h.EDITtmaxmin,'string', num2str(d.tmaxmin,3) );
set( d.h.SLIDERtmax,'value',(d.tmax - d.tmaxmin)/(d.tmaxmax - d.tmaxmin) );

%Imax
set( d.h.EDITImax,'string', num2str(d.Imax,4) );
set( d.h.EDITImaxmax,'string', num2str(d.Imaxmax,4) );
set( d.h.EDITImaxmin,'string', num2str(d.Imaxmin,4) );
set( d.h.SLIDERImax,'value',(d.Imax - d.Imaxmin)/(d.Imaxmax - d.Imaxmin) );

%eta
set( d.h.EDITeta,'string', num2str(d.eta,4) );
set( d.h.EDITetamax,'string', num2str(d.etamax,4) );
set( d.h.EDITetamin,'string', num2str(d.etamin,4) );
set( d.h.SLIDEReta,'value',(d.eta - d.etamin)/(d.etamax - d.etamin) );

%Toggles
set( d.h.CHECKSIR,'value',d.overlay_SIR  );
set( d.h.CHECK3d,'value',d.plot_3D_SSD );
set( d.h.CHECKuseSR,'value',d.use_S_and_R );
set( d.h.TOGGLEfixscale,'value', d.fix_axis_scale);
set( d.h.TOGGLER0,'value',d.fix_R0 );
set( d.h.TOGGLEeuler,'value',d.plot_euler );

%Axis scales
set( d.h.EDITtscalemin,'string', num2str(d.tscalemin,3) );
set( d.h.EDITtscalemax,'string', num2str(d.tscalemax,3) );
set( d.h.EDITymin,'string', num2str(d.ISDscalemin,3) );
set( d.h.EDITymax,'string', num2str(d.ISDscalemax,3) );

%%

%% eyam output function. Outputs from this function are returned to the command line.
function varargout = eyam_OutputFcn(hObject, eventdata, handles)
global d

% Get default command line output from handles structure
varargout{1} = d.h.output;

%%

%% Save PNG of graph pushbutton
function PUSHsavepng_Callback(hObject, eventdata, handles)
global d
d.plot_and_save_PNG = 1;
eyam_update;
d.plot_and_save_PNG = 0;

%%

%% Overlay data pushputton
function PUSHoverlaydata_Callback(hObject, eventdata, handles)
global d

%Load S,I,,R,D data from an Excel file. It is assumed all epidemics have the
%same format i.e. columns of t,I,S,R,D
[filename, pathname] = uigetfile('.xlsx','Choose epidemic SIRD data..');
if filename==0
    d.overlay_data = 0;
    d.tdata = [];
    d.Idata = [];
    d.Sdata = [];
    d.Rdata = [];
    d.Ddata = [];
else
    %Load Excel file
    [num,txt,raw] = xlsread( [pathname,filename] );
    
    %Time in months
    d.tdata = num(:,1).';
    
    %I,S,R,D poulations
    d.Idata = num(:,2).'; d.Sdata = num(:,3).'; d.Rdata = num(:,4).'; d.Ddata = num(:,5).';
    
    %If any S,R,D data is -1, this means no data.
    d.Sdata( d.Sdata ==-1 ) = NaN;
    d.Rdata( d.Rdata ==-1 ) = NaN;
    d.Ddata( d.Ddata ==-1 ) = NaN;
    
    %Set over data parameter
    d.overlay_data = 1;
    
    %Determine estimates of Imax and tmax from data
    d.Imax = max( d.Idata );
    i = find( d.Idata == max(d.Idata) );
    d.tmax = d.tdata( i(1) );
    d.Imaxmax = 1.5*d.Imax;
    d.Imaxmin = 0.5*d.Imax;
    d.tmaxmax = 1.5*d.tmax;
    d.tmaxmin = 0.5*d.tmax;
    
    %Update edit boxes and sliders
    
    %tmax
    set( d.h.EDITtmax,'string', num2str(d.tmax,3) );
    set( d.h.EDITtmaxmax,'string', num2str(d.tmaxmax,3) );
    set( d.h.EDITtmaxmin,'string', num2str(d.tmaxmin,3) );
    set( d.h.SLIDERtmax,'value',(d.tmax - d.tmaxmin)/(d.tmaxmax - d.tmaxmin) );
    
    %Imax
    set( d.h.EDITImax,'string', num2str(d.Imax,4) );
    set( d.h.EDITImaxmax,'string', num2str(d.Imaxmax,4) );
    set( d.h.EDITImaxmin,'string', num2str(d.Imaxmin,4) );
    set( d.h.SLIDERImax,'value',(d.Imax - d.Imaxmin)/(d.Imaxmax - d.Imaxmin) );
end
eyam_update;

%%

%% Default model pushbutton
function PUSHdefault_Callback(hObject, eventdata, handles)
global d

%Set defaults
eyam_defaults

%Update eyam
eyam_update;

%%

%% Fix axis scale toggle
function TOGGLEfixscale_Callback(hObject, eventdata, handles)
global d
v = get( d.h.TOGGLEfixscale,'value' );
if v==1;
    d.fix_axis_scale = 1;
else
    d.fix_axis_scale = 0;
end
eyam_update;

%%

%% Overlay S and R as well as I checkbox
function CHECKSIR_Callback(hObject, eventdata, handles)
global d
v = get( d.h.CHECKSIR,'value' );
if v==1;
    d.overlay_SIR = 1;
else
    d.overlay_SIR = 0;
end
eyam_update;

%%

%% Imax slider
function SLIDERImax_Callback(hObject, eventdata, handles)
global d

%Compute value of slider
v = get(handles.SLIDERImax,'value');

%Determine vale of Imax and update associated edit box
d.Imax = v * ( d.Imaxmax - d.Imaxmin ) + d.Imaxmin;
set(d.h.EDITImax,'string',num2str(d.Imax,4));

%Update GUI
eyam_update

%%

%% tmax slider
function SLIDERtmax_Callback(hObject, eventdata, handles)
global d

%Compute value of slider
v = get(handles.SLIDERtmax,'value');

%Determine vale of tmax and update associated edit box
d.tmax = v * ( d.tmaxmax - d.tmaxmin ) + d.tmaxmin;
set(d.h.EDITtmax,'string',num2str(d.tmax,4));

%Update GUI
eyam_update

%%

%% alpha slider
function SLIDERa_Callback(hObject, eventdata, handles)
global d

%Compute value of slider
v = get(handles.SLIDERa,'value');

%Determine vale of a and update associated edit box
d.a = v * ( d.amax - d.amin ) + d.amin;
set(d.h.EDITa,'string',num2str(d.a,4));

%Update GUI
eyam_update

%%

%% eta slider
function SLIDEReta_Callback(hObject, eventdata, handles)
global d

%Compute value of slider
v = get(handles.SLIDEReta,'value');

%Determine vale of eta and update associated edit box
d.eta = v * ( d.etamax - d.etamin ) + d.etamin;
set(d.h.EDITeta,'string',num2str(d.eta,4));

%Update GUI
eyam_update

%%

%% Imax edit box
function EDITImax_Callback(hObject, eventdata, handles)
global d

%Get numerical value of Imax
d.Imax = str2num( get(handles.EDITImax,'string') );

%Check limits and clip if necessary
if d.Imax > d.Imaxmax
    d.Imax = d.Imaxmax;
elseif d.Imax < d.Imaxmin
    d.Imax = d.Imaxmin;
end

%Update EDIT box
set(handles.EDITImax,'string',num2str( d.Imax ) );

%Update slider
set(handles.SLIDERImax,'value',(d.Imax-d.Imaxmin)/(d.Imaxmax - d.Imaxmin) );

%Update model
eyam_update

%%

%% tmax edit box
function EDITtmax_Callback(hObject, eventdata, handles)
global d

%Get numerical value of tmax
d.tmax = str2num( get(handles.EDITtmax,'string') );

%Check limits and clip if necessary
if d.tmax > d.tmaxmax
    d.tmax = d.tmaxmax;
elseif d.tmax < d.tmaxmin
    d.tmax = d.tmaxmin;
end

%Update EDIT box
set(handles.EDITtmax,'string',num2str( d.tmax ) );

%Update slider
set(handles.SLIDERtmax,'value',(d.tmax-d.tmaxmin)/(d.tmaxmax - d.tmaxmin) );

%Update model
eyam_update

%%

%% a (alpha) edit box
function EDITa_Callback(hObject, eventdata, handles)
global d

%Get numerical value of a
d.a = str2num( get(handles.EDITa,'string') );

%Check limits and clip if necessary
if d.a > d.amax
    d.a = d.amax;
elseif d.a < d.amin
    d.a = d.amin;
end

%Update EDIT box
set(handles.EDITa,'string',num2str( d.a ) );

%Update slider
set(handles.SLIDERa,'value',(d.a-d.amin)/(d.amax - d.amin) );

%Update model
eyam_update

%%

%% eta edit box
function EDITeta_Callback(hObject, eventdata, handles)
global d

%Get numerical value of eta
d.eta = str2num( get(handles.EDITeta,'string') );

%Check limits and clip if necessary
if d.eta > d.etamax
    d.eta = d.etamax;
elseif d.eta < d.etamin
    d.eta = d.etamin;
end

%Update EDIT box
set(handles.EDITeta,'string',num2str( d.eta ) );

%Update slider
set(handles.SLIDEReta,'value',(d.eta-d.etamin)/(d.etamax - d.etamin) );

%Update model
eyam_update

%%

%% Imaxmax edit box
function EDITImaxmax_Callback(hObject, eventdata, handles)
global d

%Get numerical value of Imaxmax
d.Imaxmax = str2num( get(handles.EDITImaxmax,'string') );

%Get new value of slider, and clip if v > 1
v = (d.Imax-d.Imaxmin)/(d.Imaxmax - d.Imaxmin);
if v>1
    v=1;
    d.Imax = d.Imaxmax;
    set(handles.EDITImax,'string',num2str( d.Imax ) );
end

%Update slider
set(handles.SLIDERImax,'value',v );

%Update model
eyam_update

%%

%% Imaxmin edit box
function EDITImaxmin_Callback(hObject, eventdata, handles)
global d

%Get numerical value of Imaxmin
d.Imaxmin = str2num( get(handles.EDITImaxmin,'string') );

%Get new value of slider, and clip if v < 0
v = (d.Imax-d.Imaxmin)/(d.Imaxmax - d.Imaxmin);
if v<0
    v=0;
    d.Imax = d.Imaxmin;
    set(handles.EDITImax,'string',num2str( d.Imax ) );
end

%Update slider
set(handles.SLIDERImax,'value',v );

%Update model
eyam_update

%%

%% tmaxmax edit box
function EDITtmaxmax_Callback(hObject, eventdata, handles)
global d

%Get numerical value of tmaxmax
d.tmaxmax = str2num( get(handles.EDITtmaxmax,'string') );

%Get new value of slider, and clip if v > 1
v = (d.tmax-d.tmaxmin)/(d.tmaxmax - d.tmaxmin);
if v>1
    v=1;
    d.tmax = d.tmaxmax;
    set(handles.EDITtmax,'string',num2str( d.tmax ) );
end

%Update slider
set(handles.SLIDERtmax,'value',v );

%Update model
eyam_update

%%

%% tmaxmin edit box
function EDITtmaxmin_Callback(hObject, eventdata, handles)
global d

%Get numerical value of tmaxmin
d.tmaxmin = str2num( get(handles.EDITtmaxmin,'string') );

%Get new value of slider, and clip if v < 0
v = (d.tmax-d.tmaxmin)/(d.tmaxmax - d.tmaxmin);
if v<0
    v=0;
    d.tmax = d.tmaxmin;
    set(handles.EDITtmax,'string',num2str( d.tmax ) );
end

%Update slider
set(handles.SLIDERtmax,'value',v );

%Update model
eyam_update

%%

%% amax edit box
function EDITamax_Callback(hObject, eventdata, handles)
global d

%Get numerical value of amax
d.amax = str2num( get(handles.EDITamax,'string') );

%Get new value of slider, and clip if v > 1
v = (d.a-d.amin)/(d.amax - d.amin);
if v>1
    v=1;
    d.a = d.amax;
    set(handles.EDITa,'string',num2str( d.a ) );
end

%Update slider
set(handles.SLIDERa,'value',v );

%Update model
eyam_update

%%

%% amin edit box
function EDITamin_Callback(hObject, eventdata, handles)
global d

%Get numerical value of amin
d.amin = str2num( get(handles.EDITamin,'string') );

%Get new value of slider, and clip if v < 0
v = (d.a-d.amin)/(d.amax - d.amin);
if v<0
    v=0;
    d.a = d.amin;
    set(handles.EDITa,'string',num2str( d.a ) );
end

%Update slider
set(handles.SLIDERa,'value',v );

%Update model
eyam_update

%%

%% etamax edit box
function EDITetamax_Callback(hObject, eventdata, handles)
global d

%Get numerical value of etamax
d.etamax = str2num( get(handles.EDITetamax,'string') );

%Make sure it is less than 0.99
if d.etamax > 0.99
    d.etamax = 0.99;
    set( handles.EDITetamax,'string',num2str(d.etamax) )
end

%Get new value of slider, and clip if v > 1
v = (d.eta-d.etamin)/(d.etamax - d.etamin);
if v>1
    v=1;
    d.eta = d.etamax;
    set(handles.EDITeta,'string',num2str( d.eta ) );
end

%Update slider
set(handles.SLIDEReta,'value',v );

%Update model
eyam_update

%%

%% etamin edit box
function EDITetamin_Callback(hObject, eventdata, handles)
global d

%Get numerical value of etamin
d.etamin = str2num( get(handles.EDITetamin,'string') );

%Make sure it is more than 0.01
if d.etamin < 0.01
    d.etamin = 0.01;
    set( handles.EDITetamin,'string',num2str(d.etamin) )
end

%Get new value of slider, and clip if v < 0
v = (d.eta-d.etamin)/(d.etamax - d.etamin);
if v<0
    v=0;
    d.eta = d.etamin;
    set(handles.EDITeta,'string',num2str( d.eta ) );
end

%Update slider
set(handles.SLIDEReta,'value',v );

%Update model
eyam_update

%%

%% tscalemin edit box
function EDITtscalemin_Callback(hObject, eventdata, handles)
global d

%Get numerical value of tscalemin
d.tscalemin = str2num( get(handles.EDITtscalemin,'string') );
set( d.h.AXESeyam, 'xlim', [d.tscalemin,d.tscalemax] );

%%

%% tscalemax edit box
function EDITtscalemax_Callback(hObject, eventdata, handles)
global d

%Get numerical value of tscalemin
d.tscalemax = str2num( get(handles.EDITtscalemax,'string') );
set( d.h.AXESeyam, 'xlim', [d.tscalemin,d.tscalemax] );

%%

%% Graph y scale min edit box
function EDITymin_Callback(hObject, eventdata, handles)
global d

%Get numerical value of ymax and update graph
d.ISDscalemin = str2num( get(handles.EDITymin,'string') );
set( d.h.AXESeyam, 'ylim', [d.ISDscalemin,d.ISDscalemax] );

%%

%% Graph y scale max edit box
function EDITymax_Callback(hObject, eventdata, handles)
global d

%Get numerical value of ymax and update graph
d.ISDscalemax = str2num( get(handles.EDITymax,'string') );
set( d.h.AXESeyam, 'ylim', [d.ISDscalemin,d.ISDscalemax] );

%%

%% SSD surface 3D toggle
function CHECK3d_Callback(hObject, eventdata, handles)
global d
if get( d.h.CHECK3d,'value')==1
    d.plot_3D_SSD=1;
else
    d.plot_3D_SSD=0;
end

%%

%% Use S and R for SSD calculation checkbox
function CHECKuseSR_Callback(hObject, eventdata, handles)
global d
if get( d.h.CHECKuseSR,'value')==1
    d.use_S_and_R=1;
else
    d.use_S_and_R=0;
end
eyam_update

%%

%% Optimize Imax, tmax by working out an SSD surface and
%  detmining the (tmax,Imax) coordinate of the minimum
function PUSHoptimize_Callback(hObject, eventdata, handles)
global d

%Array size for SSD plot
N = 20;

%Construct arrays of Imax and tmax, and initialize SSD array
tmax = linspace( d.tmaxmin, d.tmaxmax, N );
Imax = linspace( d.Imaxmin, d.Imaxmax, N );
[tmax,Imax] = meshgrid(tmax,Imax);
SSD = zeros(N,N);

%Run model for each scenario
for n=1:N^2
    [S,I,R,t,rho,N,R0,SSD(n)] = eyam_model( tmax(n), Imax(n), d.a,...
        d.eta, d.Ni, d.tdata, d.Idata,d.Sdata,d.Rdata, d.use_S_and_R );
end

%Find tmax,Imax which correspond to minimum SSD
SSDmin = min(min(SSD));
[i,j] = find( SSD==SSDmin );
i =i(1); j = j(1);
tmaxbest = tmax(i,j);
Imaxbest = Imax(i,j);

%Assign Imax and tmax to these best values
d.Imax = Imaxbest;
d.tmax = tmaxbest;

%Update GUI
set( d.h.EDITImax,'string',num2str(d.Imax) );
set( d.h.EDITtmax,'string',num2str(d.tmax) );
set( d.h.SLIDERImax,'value',(d.Imax - d.Imaxmin)/(d.Imaxmax - d.Imaxmin) );
set( d.h.SLIDERtmax,'value',(d.tmax - d.tmaxmin)/(d.tmaxmax - d.tmaxmin) );
eyam_update

%Plot SSD surface
fig = figure('name','SSD surface','color',[1 1 1] );
surf( tmax, Imax, SSD, SSD ); hold on;
plot3( tmaxbest, Imaxbest, SSDmin,'m*','markersize',d.msize );
shading interp
set( gca,'fontsize', d.fsize );
axis tight
if d.plot_3D_SSD==1
    %axis vis3d
    view([142,48]);
else
    view(2);
    box on
end
colorbar('fontsize', d.fsize );
interp_colormap(1000);
xlabel('tmax','fontsize',d.fsize);
ylabel('Imax','fontsize',d.fsize);
title({['SSD surface for \eta=',num2str(d.eta),', \alpha=',num2str(d.a)],...
    ['Best Imax=',num2str(Imaxbest,4),', Best tmax=',num2str(tmaxbest,3)]})
png_filename = [ 'eyam SSD a=',strrep(num2str(d.a,4),'.','p'),' Imax=',...
    strrep(num2str(d.Imax,4),'.','p'),' tmax=',strrep(num2str(d.tmax,3),'.','p'),...
    ' eta=',strrep(num2str(d.eta,3),'.','p'),' N=',strrep(num2str(N,5),'.','p'),'.png'];
if d.plot_3D_SSD==1
    png_filename = ['3D ',png_filename];
end
print( fig,png_filename,'-dpng','-r300');
close(fig);

%%

%% Optimize a, eta by working out an SSD surface and
%  detmining the (a,eta) coordinate of the minimum
function PUSHoptimizeaeta_Callback(hObject, eventdata, handles)
global d

%Array size for SSD plot
N = 20;

%Construct arrays of a and eta, and initialize SSD array
a = linspace( d.amin, d.amax, N );
eta = linspace( d.etamin, d.etamax, N );
[a,eta] = meshgrid(a,eta);
SSD = zeros(N,N);

%Run model for each scenario
for n=1:N^2
    [S,I,D,t,rho,N,R0,SSD(n)] = eyam_model( d.tmax, d.Imax, a(n),...
        eta(n), d.Ni, d.tdata, d.Idata, d.Sdata, d.Rdata, d.use_S_and_R  );
end

%Find a,eta which correspond to minimum SSD
SSDmin = min(min(SSD));
[i,j] = find( SSD==SSDmin );
i =i(1); j = j(1);
abest = a(i,j);
etabest = eta(i,j);

%Assign eta and a to these best values
d.eta = etabest;
d.a = abest;

%Update GUI
set( d.h.EDITeta,'string',num2str(d.eta) );
set( d.h.EDITa,'string',num2str(d.a) );
set( d.h.SLIDERa,'value',(d.a - d.amin)/(d.amax - d.amin) );
set( d.h.SLIDEReta,'value',(d.eta - d.etamin)/(d.etamax - d.etamin) );
eyam_update

%Plot SSD surface
fig = figure('name','SSD surface','color',[1 1 1] );
surf( a, eta, SSD, SSD ); hold on;
plot3( abest, etabest, SSDmin,'m*','markersize',d.msize );
shading interp
set( gca,'fontsize', d.fsize );
axis tight
if d.plot_3D_SSD==1
    %axis vis3d
    view([142,48]);
else
    view(2);
    box on
end
colorbar('fontsize', d.fsize );
interp_colormap(1000);
xlabel('a','fontsize',d.fsize);
ylabel('eta','fontsize',d.fsize);
title({['SSD surface for tmax=',num2str(d.tmax),', Imax=',num2str(d.Imax)],...
    ['Best eta=',num2str(etabest,4),', Best a=',num2str(abest,3)]})
png_filename = [ 'eyam SSD a=',strrep(num2str(d.a,4),'.','p'),' eta=',...
    strrep(num2str(d.eta,4),'.','p'),' tmax=',strrep(num2str(d.tmax,3),'.','p'),...
    ' Imax=',strrep(num2str(d.Imax,3),'.','p'),' N=',strrep(num2str(N,5),'.','p'),'.png'];
if d.plot_3D_SSD==1
    png_filename = ['3D ',png_filename];
end
print( fig,png_filename,'-dpng','-r300');
close(fig);

%%

%Fix R = 0 at t=0 e.g. to compare to Mompesson data
function TOGGLER0_Callback(hObject, eventdata, handles)
global d
if get( d.h.TOGGLER0,'value' ) == 1
    d.fix_R0 = 1;
else
    d.fix_R0 = 0;
end
eyam_update

%%

%Overlay Euler method results
function TOGGLEeuler_Callback(hObject, eventdata, handles)
global d
if get( d.h.TOGGLEeuler,'value' ) == 1
    d.plot_euler = 1;
else
    d.plot_euler  = 0;
end
eyam_update

%%

%Save .mat file of data array d
function PUSHsavemat_Callback(hObject, eventdata, handles)
global d
filename = ['eyam ',strrep( datestr(now),':','-' )];
save( filename,'d' );

%%

%Load .mat file containing data array d
function PUSHloadmat_Callback(hObject, eventdata, handles)
global d
[filename, pathname] = uigetfile('.mat','Choose eyam GUI settings in.mat file..');
if filename~=0
    %Only update the data, not the GUI handles.
    h = d.h; load([pathname,filename]); d.h = h;
    update_gui_items
    eyam_update
end

%%

%Update GUI
function eyam_update
global d

%Run Eyam model
[d.S,d.I,d.R,d.t,d.rho,d.N,d.R0,d.SSD] = eyam_model( d.tmax, d.Imax, d.a,...
    d.eta, d.Ni, d.tdata, d.Idata, d.Sdata, d.Rdata, d.use_S_and_R  );

%Find I0, S0
d.I0 = interp1(d.t,d.I,0,'spline','extrap' );
d.S0 = interp1(d.t,d.S,0,'spline','extrap' );

%Run Euler solvers, if requested
if d.plot_euler==1
    [d.Se,d.Ie,d.Re,d.te] = euler( d.S0,d.I0, d.a, d.a/rho, d.dt, d.t(end) );
    tei = linspace( d.te(1), d.te(end), d.N_euler );
    d.Ie = interp1( d.te, d.Ie, d.tei );
    d.Se = interp1( d.te, d.Se, d.tei );
    d.Re = interp1( d.te, d.Re, d.tei );
    d.te = tei;
end

%Run K&K approximate solver
[d.Skk,d.Ikk,d.Rkk,d.Nkk,d.R0kk] = K_and_K_eyam_model( d.t, d.rho, d.Imax, d.tmax, d.a );

%Fix R=0 at t=0? This should correct for the offset between Euler solver
%and the others. Euler assumes R=0 at t=0;
if d.fix_R0 == 1
    d.RR0 = interp1( d.t, d.R, 0,'spline','extrap'  );
    d.R = d.R - d.RR0;
    d.RR0 = interp1( d.t, d.Rkk, 0,'spline','extrap'  );
    d.Rkk = d.Rkk - d.RR0;
end

%Update GUI with derived parameters
set( d.h.TEXTR0,'string',['R0 = ',num2str(d.R0,5)] );
set( d.h.TEXTrho,'string',['rho = ',num2str(d.rho,5)] );
set( d.h.TEXTN,'string',['N = ',num2str(d.N,5)] );
set( d.h.TEXTbeta,'string',['beta = ',num2str(d.a/d.rho,5)] );
set( d.h.TEXTSSD,'string',['SSD = ',num2str(d.SSD,5)] );

%Where to update plot? GUI, or new figure window to make a PNG output.
if d.plot_and_save_PNG==1
    %Create new figure
    fig = figure;
    figure(fig);
    fsize = 18;
else
    %Point to axes
    figure( d.h.FIGUREeyam );
    axes( d.h.AXESeyam );
    fsize = d.fsize;
    cla;
end

%Plot Eyam model and overlay S,I,D vs t data
plot( d.t,d.I,'r-','linewidth',2); hold on;
if d.overlay_SIR==1
    plot( d.t,d.S,'g-','linewidth',2);
    plot( d.t,d.R,'b-','linewidth',2);
    legend({'I','S','R'},'fontsize',d.fsize);
    plot( d.t,d.Skk,'g--','linewidth',2);
    plot( d.t,d.Rkk,'b--','linewidth',2);
else
    legend({'I'},'fontsize',d.fsize);
end
plot( d.t,d.Ikk,'r--','linewidth',1);
if d.plot_euler==1
    plot( d.te,d.Ie,'ro');
    plot( d.te,d.Se,'go','linewidth',2);
    plot( d.te,d.Re,'bo','linewidth',2);
end
if d.overlay_data==1
    plot( d.tdata, d.Idata,'r+', 'markersize', d.msize );
    if d.overlay_SIR==1
        plot( d.tdata, d.Sdata,'g+', 'markersize', d.msize );
        plot( d.tdata, d.Rdata,'b+', 'markersize', d.msize );
        plot( d.tdata, d.Ddata,'k+', 'markersize', d.msize );
    end
end
xlabel( 't /months', 'fontsize', fsize );
if d.overlay_SIR==1
    ylabel( 'S,I,R population', 'fontsize', fsize );
else
    ylabel( 'Infective (I) population', 'fontsize', fsize );
end
title( {'Eyam model fit',...
    ['N=',num2str(d.N,4),', I_{max}=',num2str(d.Imax,4),', R_0=',...
    num2str(d.R0,3),', t_{max}=',num2str(d.tmax,3) ],...
    ['\alpha=',num2str(d.a,4),', \rho=',num2str(d.rho,4),', \eta=',num2str(d.eta,4),...
    ', S_0=',num2str(d.S0,4),', I_0=',num2str(d.I0,4)]},...
    'fontsize', fsize )
set( gca,'fontsize', fsize );
grid on
if d.fix_axis_scale == 1
    xlim( [d.tscalemin,d.tscalemax] );
    ylim( [d.ISDscalemin,d.ISDscalemax] );
else
    axis tight
    ylimits = get( gca, 'ylim' );
    set( gca, 'ylim', [0,ylimits(2)] );
    d.ISDscalemin = 0;
    d.ISDscalemax = ylimits(2);
    xlimits = get( gca, 'xlim' );
    d.tscalemin = xlimits(1);
    d.tscalemax = xlimits(2);
    set( d.h.EDITtscalemin,'string',num2str(d.tscalemin,3) );
    set( d.h.EDITtscalemax,'string',num2str(d.tscalemax,3) );
    set( d.h.EDITymax,'string',num2str(d.ISDscalemax,3) );
    set( d.h.EDITymin,'string',num2str(d.ISDscalemin,3) );
end

%Save externally plotted graph, if option set
if d.plot_and_save_PNG==1
    png_filename = [ 'eyam a=',strrep(num2str(d.a,4),'.','p'),' Imax=',...
        strrep(num2str(d.Imax,3),'.','p'),' tmax=',strrep(num2str(d.tmax,5),'.','p'),...
        ' eta=',strrep(num2str(d.eta,3),'.','p'),' N=',strrep(num2str(d.N,5),'.','p'),'.png'];
    print( fig,png_filename,'-dpng','-r300');
    close(fig);
    figure(d.h.FIGUREeyam); axes(d.h.AXESeyam);
end

%%

%figcenter
% Centres a figure with handle fig in the current display.
function figcentre(fig)

%Get screen dimensions in pixels.
set(0,'units','pixels') ;
screen_size=get(0,'Screensize');

%Get original figure units
orig_units = get(fig,'units');

%Get position vector for figure in pixels
set(fig,'units','pixels');
fig_position=get(gcf,'position') ;

%Define coordinates or figure bottom hand corner from screen bottom hand corner
%that will place the figure window in the center of the screen.
centre_x = 0.5*screen_size(3) - 0.5*fig_position(3);
centre_y = 0.5*screen_size(4) - 0.5*fig_position(4);

%Modify figure position to centre
set(fig,'position',[centre_x,centre_y,fig_position(3),fig_position(4)] );

%Reset figure units
set(fig,'units',orig_units);

%%

%tor_of_z_cspline
% Evaluates numeric integral of tor(z) using cubic cpline fit
function [z,tor] = tor_of_z_cspline( zminus,zplus, Ni, xmax )
dz = 0.01;
z = linspace( zminus + dz, zplus - dz, Ni );
f = 1./( xmax + 1 - exp(-z) - z );
tor = integrator( z, f, 0, z );

%%

%tor_of_z
% Evaluates numeric integral of tor(z).
function [z,tor] = tor_of_z( zminus,zplus, Ni, xmax )
dz = 0.01;
z_pos = linspace( 0, zplus - dz, Ni );
z_neg = linspace( 0, zminus + dz, Ni );
f_pos = 1./( xmax + 1 - exp(-z_pos) - z_pos );
f_neg = 1./( xmax + 1 - exp(-z_neg) - z_neg );
tor_pos = trapezium_rule( z_pos, f_pos );
tor_neg = trapezium_rule( z_neg, f_neg );
z = [fliplr(z_neg),z_pos];
tor = [fliplr(tor_neg),tor_pos];

%%

%trapezium_rule
% Integration via the trapezium rule. x is a equally spaced vector, and y
% are the corresponding integrand values. The integration is from x(1) to
% the values in x. Output vector I has the same dimensions as x.
function I = trapezium_rule( x, y )
dx = x(2)-x(1);
I = dx*( cumsum(y) - y(1)/2 - y(end)/2 );

%%

%integrator
% Integration function using cubic splines.
% I = integrator( xx, yy, x0, x )
%
% xx, yy     Samples from function y(x) which is being integrated. These
%            must be the same dimensions and have at least three elements each.
% x0         Starting point (i.e. lower limit) of the integral
% x          Upper limit of the integral. This can be an array of any
%            dimensions.
%
% I          Output of integral corresponindg to array x
%
% The function is based on the code written on pages 292 and 294
% of "Mastering Matlab 6" by Duane Hanselman & Bruce Littlefield.
function I = integrator( xx, yy, x0, x )

%Check inputs are the right size
if ( numel(xx) == numel(yy) ) && ( numel(xx) >= 3 )
    
    %Sort xx in ascending order
    [xx,i] = sort(xx); yy = yy(i);
    
    %Compute piecewise polynomial based upon cubic splines.
    pp=spline(xx,yy);
    
    %Take apart piecewise polynomial
    [br,co,npy,nco] = unmkpp(pp);
    
    %Scale factors for integration
    sf=nco:-1:1;
    
    %Integral coefficients
    ico=[co./sf(ones(npy,1),:) zeros(npy,1)];
    
    %Integral spline has higher order
    nco=nco+1;
    
    %Set integration constant of all splines to be zero
    ico(1,nco)=0;
    
    %Find constant terms in polynomials
    for k=2:npy
        ico(k,nco)=polyval(ico(k-1,:),br(k)-br(k-1));
    end
    
    %Build pp form for integral
    ppi=mkpp(br,ico);
    
    %Evaluate definite integral between limits of x0 to x
    I = ppval(ppi,x) - ppval(ppi,x0);
    
else
    I = NaN(size(x));
end

%%

%interp_colormap
% Function which interpolates current colourmap to yield better graduated
% shading. N is number of possible colours.
function interp_colormap(N)

%Get current colourmap
map = colormap;

%Initialise new colormap
new_map = ones(N,3);

%Get size of current colormap and initalise red,green,blue vectors
dim = size(map);
R = ones(1,dim(1));
G = ones(1,dim(1));
B = ones(1,dim(1));
RR = ones(1,N);
GG = ones(1,N);
BB = ones(1,N);

%Populate these with current colormap
R(:) = map(:,1);
G(:) = map(:,2);
B(:) = map(:,3);

%Interpolate to yield new colour map
x = linspace( 1, dim(1), N );
RR = interp1( 1:dim(1), R, x );
GG = interp1( 1:dim(1), G, x );
BB = interp1( 1:dim(1), B, x );
new_map(:,1) = RR(:);
new_map(:,2) = GG(:);
new_map(:,3) = BB(:);

%Set colormap to be new map
colormap( new_map );

%End of code






