% Model of the Black Death plague outbreak in the English village of Eyam
% using data recorded by Mompesson in 1666.
%
% Times are in months ( 1 month is defined as 365/12 days )
% I mean population of Eyam that are Infected
% S means population of Eyam that are Succeptable to infection. In this model
% it is assumed there is no natural immunity.
% D means population that have died since t=0
% N = S + I is the population at time t
% N0 = S + I + D is the population at time t=0
%
% Based upon a Div project by Dr John Cullerne, Winchester College.
%
% LAST UPDATED by Andy French December 2018

function eyam_deterministic

%% INPUTS %%

%Define timestep
dt = 0.005;

%Define maximum time t value /months
tmax = 4;

%Define range and number of alpha iterations
alphamin = 2.7;
alphamax = 3.1;
num_alpha = 100;

%Fontsize and markersize for plots
fsize = 22;
msize = 14;

%%

%Display greeting message and start clock
disp( ' EYAM PLAGUE DETERMINISTIC EPIDEMIOLOGICAL MODEL ')
disp( ' Dr Andrew French & Dr John Cullerne. January 2019 ' )
disp(' ');
disp( ' Plotting Mompesson Eyam data ... ' )
t0 = tic;

% Mompesson Eyam data
t_data = [0,0.51,1.02,1.53,2.04,2.55,3.57];
I_data = [14.5,22,29,21,8,8,0];
S_data = [235,201,153.5,121,108,97,83];
I0 = I_data(1);
S0 = S_data(1);

%Determine derived parameters
N = length(t_data);
D_data = zeros(1,N);
x =  zeros(1,N);
y =  zeros(1,N);
for n=1:N
    D_data(n) = I0 + S0 - I_data(n) - S_data(n);
    x(n) = log( S0/S_data(n) );
    y(n) = I0 + S0 - I_data(n) - S_data(n);
end

%Plot Mompesson's Eyam data
fig = figure('color',[1 1 1],'name','Mompesson Eyam data');
plot( t_data, I_data,'r-','linewidth',2 ); hold on;
plot( t_data, S_data,'g-','linewidth',2);
plot( t_data, D_data,'b-','linewidth',2);
plot( t_data, I_data,'r+', 'markersize', msize );
plot( t_data, S_data,'g+', 'markersize', msize );
plot( t_data, D_data,'b+', 'markersize', msize );
xlabel( 't /months', 'fontsize', fsize );
ylabel( 'Eyam population', 'fontsize', fsize );
title( 'Mompesson Eyam data', 'fontsize', fsize );
set( gca,'fontsize', fsize );
legend( {'I','S','D'}, 'fontsize', fsize );
xlim( [0, tmax]  ); grid on

%Export PNG file
print('Mompesson data.png', '-r300','-dpng' ); close(fig);

%

%Determining Eyam model parameter ratio alpha/beta
%from I and S data
disp(' Determining Eyam model parameter ratio alpha/beta from I and S data ...' )

%Determine line of best fit through x,y values formed from Eyam
%I and S data
[m,yfit,xfit,dm,r] = best_fit_m( x,y );
alpha_over_beta = m;  alpha_over_beta_error = dm;

%Plot best fit to Eyam data to determine alpha/beta ratio
fig = figure('color',[1 1 1],'name','alpha/beta ratio');
plot( xfit,yfit,'k-','linewidth',2 ); hold on;
plot( x,y,'r+','markersize', msize  );
xlabel( 'ln( S_0/S )', 'fontsize', fsize );
ylabel( 'I_0 + S_0 - I - S', 'fontsize', fsize );
title( ['\alpha/\beta = (',num2str( alpha_over_beta,3 ),...
    ' +/- ',num2str(alpha_over_beta_error,3),')'], 'fontsize', fsize )
legend( {'Linear fit','Data'},'fontsize', fsize,'location','northwest' )
set( gca,'fontsize', fsize ); xlim( [0, 1.1] ); ylim( [0, 175] ); grid on

%Export PNG file
print('Mompesson alpha over beta calc.png', '-r300','-dpng' );
close(fig);

%

%Eyam model
disp(' ')
disp( ' Running Eyam model for a range of alpha values ...' )

%Compute number of data points i.e. xmax = x0 + (N-1)*dx

%Run model in a loop and determine alpha parameter which minimizes the sum
%of square differences (SSD) from the Eyam data
alpha = linspace(alphamin,alphamax,num_alpha);
SSD = zeros(1,num_alpha);
for i=1:num_alpha
    
    %Display message to console window
    disp( [' Alpha iteration number = ',num2str(i),' of ',num2str(num_alpha)] )
    
    %Run Eyam model
    beta = alpha(i)/alpha_over_beta;
    [t, I, S, D] = eyam_model( dt, I0, S0, alpha(i), beta, tmax );
    
    %Interpolate at Eyam data times
    Ii = interp1( t, I, t_data );
    Si = interp1( t, S, t_data );
    Di = interp1( t, D, t_data );
    
    %Determine sum of square differences between model and data
    SSD(i) = sum( (I_data - Ii).^2 + (S_data - Si).^2 + (D_data - Di).^2 );
end

%Find alpha parameter which corresponds to minimum SSD
imin = find( SSD == min(SSD) );
amin = alpha( imin(1) );

%Plot SSD vs alpha
fig = figure('color',[1 1 1],'name','SSD vs alpha');
plot( alpha,SSD,'r-','linewidth',2 ); hold on;
plot( amin,min(SSD),'+','markersize', msize );
ylimits = get(gca,'ylim');
xlabel( 'alpha', 'fontsize', fsize );
ylabel( 'Sum of square differences', 'fontsize', fsize );
title( ['\alpha = ',num2str(amin,3), ' for minimum SSD '], 'fontsize', fsize )
set( gca,'fontsize', fsize );
plot( [alphamin, alphamax],min(SSD)*[1,1],'k--','linewidth',2 ); 
plot( amin*[1,1],get(gca,'ylim'),'k--','linewidth',2 ); 
ylim(ylimits);
xlim( [alphamin, alphamax] );
grid on

%Export PNG file
print('SSD vs alpha.png', '-r300','-dpng' );
close(fig);

%

%Eyam model, with alpha corresponding to minimum FSD, with Mompesson data overlay
disp(' ')
disp(' Computing optimal Eyam model and plotting it with Mompesson data overlay ')

%Solve Eyam model (Euler method)
alpha = amin;
beta = alpha/alpha_over_beta;
[t, I, S, D] = eyam_model( dt, I0, S0, alpha, beta, tmax );

%Plot Eyam model and overlay Mompesson data
fig = figure('color',[1 1 1],'name','Eyam model + data');
plot( t,I,'r-','linewidth',2); hold on;
plot( t,S,'g-','linewidth',2);
plot( t,D,'b-','linewidth',2);
plot( t_data, I_data,'r+', 'markersize', msize );
plot( t_data, S_data,'g+', 'markersize', msize );
plot( t_data, D_data,'b+', 'markersize', msize );
xlabel( 't /months', 'fontsize', fsize );
ylabel( 'Eyam population', 'fontsize', fsize );
title( ['Eyam Euler model: \alpha = ',num2str( alpha,3),...
    ', \beta = ',num2str(beta,3),', \Deltat = ',num2str(dt)], 'fontsize', fsize )
set( gca,'fontsize', fsize );
legend( {'I','S','D'}, 'fontsize', fsize );
xlim( [0, tmax]  )
ylim( [0, S0 + I0] )
grid on

%Export PNG file
print(['Eyam deterministic model dt=',strrep(num2str(dt),'.','p'),'.png'], '-r300','-dpng' );
close(fig);

%Display completion message
disp(' ')
disp([' Eyam model completed in ',num2str(toc(t0)),'s.' ] );
disp(' ');

%%

%Euler method solver for differential equations which
%describe model of Eyam epidemic.
function [t,I,S,D] = eyam_model( dt, I0, S0, alpha, beta, tmax )

%Initialize output vectors for t,I,S,D
t = 0 : dt : tmax;
N = length(t);
S = S0*ones(1,N);
I = I0*ones(1,N);
D = zeros(1,N);

%Loop through vectors to compute t, I, S, D.
%using the Euler first order differential equation method
for n=2:N
    t(n) = t(n-1) + dt;
    I(n) = I(n-1) + dt*( beta*S(n-1)*I(n-1) - alpha*I(n-1) );
    S(n) = S(n-1) - dt*beta*S(n-1)*I(n-1);
    D(n) = D(n-1) + dt*alpha*I(n-1);
end

%%

%best_fit_m
% Work out a line of best fit of the form y = mx to (x,y) data
function [m,yfit,xfit,dm,r] = best_fit_m( x,y )

%Compute mean averages and variances
xbar = mean(x); ybar = mean(y); 
Vx = mean( x.^2 ) - xbar^2;
Vy = mean( y.^2 ) - ybar^2;
cov_xy = mean( x.*y ) - xbar*ybar;

%Determine m
m = mean( x.*y )/mean( x.^2 );

%Product moment correlation coefficient
r = cov_xy/sqrt( Vx*Vy );

%Errors in m
N = numel(x); %Number of elements in array x
s = sqrt( (1/(N-1))*sum( (y - m*x ).^2 ) );
dm = s/( sqrt(N)*sqrt(Vx) );

%Define the line of best fit itself
xfit = linspace( min(x), max(x), 1000 );
yfit = m*xfit ;

%End of code