%eyam_stochastic
% Computes a matrix of probabilities for S,I,D populations vs time for the
% Eyam outbreak. The deterministic S,I,D equations are used in discrete
% timesteps to calculate Poisson distribution means for the random, integer
% changes to S,I,D populations.
%
% LAST UPDATED by Andy French June 2019

function eyam_stochastic

%% Eyam model inputs %%

%Number of stochastic iterations
stoc_iter= 200;

%Eyam deterministic S,I,D equation parameters
alpha = 2.99;
beta = 0.0183;

%Timestep /months
dt = 0.005;

%Define maximum time t value /months
tmax = 4;

%Multiplier of Poisson mean representing maximum population change
%per time step
lambda_multiplier = 3;

%Fontsize and markersize for plots
fsize = 22; msize = 14;

%Plot graph?
plot_graph = 1;

% Mompesson Eyam data
t_data = [0,0.51,1.02,1.53,2.04,2.55,3.57];
I_data = [14.5,22,29,21,8,8,0];
S_data = [235,201,153.5,121,108,97,83];
I0 = I_data(1); S0 = S_data(1);
D_data = S0+I0 - I_data - S_data;

%

%Display greeting message and start clock
disp( ' EYAM PLAGUE STOCHASTIC EPIDEMIOLOGICAL MODEL ')
disp( ' Dr Andrew French & Dr John Cullerne. June 2019 ' )
disp(' ');
t0 = tic;

%Eyam model, with alpha corresponding to minimum FSD, with Mompesson data overlay
disp(' Computing deterministic Eyam model')

%Solve Eyam model (Euler method)
[t, I, S, D] = eyam_deterministic_model( dt, I0, S0, alpha, beta, tmax );

%Solve Eyam model (stochastic method)
tts = []; IIs = []; SSs = []; DDs = [];
for n=1:stoc_iter
    disp([' Computing Eyam stochastic model: iteration ',num2str(n),' of ',num2str(stoc_iter)]);
    [ts, Is, Ss, Ds] = eyam_stochastic_model( dt, I0, S0, alpha, beta, tmax,lambda_multiplier );
    tts = [tts,ts.']; IIs = [IIs,Is.']; SSs = [SSs,Ss.']; DDs = [DDs,Ds.'];
end

%Work out average stochastic I,S,D
ts = sum(tts,2)/stoc_iter; Is = sum(IIs,2)/stoc_iter;
Ss = sum(SSs,2)/stoc_iter; Ds = sum(DDs,2)/stoc_iter;

%Plot Eyam model and overlay Mompesson data
if plot_graph==1
    fig = figure('color',[1 1 1],'name','Eyam model + data');
    plot( ts,Is,'r-','linewidth',2); hold on;
    plot( ts,Ss,'g-','linewidth',2);
    plot( ts,Ds,'b-','linewidth',2);
    plot( tts,IIs,'r.','markersize',msize/7);
    plot( tts,SSs,'g.','markersize',msize/7);
    plot( tts,DDs,'b.','markersize',msize/7);
    plot( ts,Is,'r-','linewidth',2); hold on;
    plot( ts,Ss,'g-','linewidth',2);
    plot( ts,Ds,'b-','linewidth',2);
    plot( t,I,'k--','linewidth',2);
    plot( t,S,'k--','linewidth',2);
    plot( t,D,'k--','linewidth',2);
    plot( t_data, I_data,'ko', 'markersize', msize );
    plot( t_data, S_data,'ko', 'markersize', msize );
    plot( t_data, D_data,'ko', 'markersize', msize );
    xlabel( 't /months', 'fontsize', fsize );
    ylabel( 'Eyam population', 'fontsize', fsize );
    title( ['Eyam model: \alpha=',num2str( alpha,3),...
        ', \beta=',num2str(beta,3),', \Deltat=',num2str(dt)], 'fontsize', fsize )
    set( gca,'fontsize', fsize );
    legend( {'I','S','D'}, 'fontsize', fsize );
    xlim( [0, tmax]  )
    ylim( [0, S0 + I0] )
    grid on
    
    %Export PNG file
    print(['Stochastic eyam model dt=',strrep(num2str(dt),'.','p'),'.png'], '-r300','-dpng' );
    close(fig);
end

%Save data
a = alpha; b = beta; clear alpha beta
save eyam_stochastic

%Display completion message
disp(' ')
disp([' Eyam model completed in ',num2str(toc(t0)),'s.' ] );
disp(' ');

%%

%Euler method solver for differential equations which
%describe model of Eyam epidemic.
function [t,I,S,D] = eyam_deterministic_model( dt, I0, S0, alpha, beta, tmax )

%Initialize output vectors for t,I,S,D
t = 0 : dt : tmax;
N = length(t);
S = S0*ones(1,N);
I = I0*ones(1,N);
D = zeros(1,N);

%Loop through vectors to compute t, I, S, D.
%using the Euler first order differential equation method
for n=2:N
    t(n) = t(n-1) + dt;
    I(n) = I(n-1) + dt*( beta*S(n-1)*I(n-1) - alpha*I(n-1) );
    S(n) = S(n-1) - dt*beta*S(n-1)*I(n-1);
    D(n) = D(n-1) + dt*alpha*I(n-1);
end

%%

%JPC's original (Excel) stochastic model of Eyam SID model
function [t,I,S,D] = orig_eyam_stochastic_model( dt, I0, S0, alpha, beta, tmax )

%Initialize output vectors for t,I,S,D
t = 0 : dt : tmax;
N = length(t);
S = S0*ones(1,N);
I = I0*ones(1,N);
D = zeros(1,N);

%Loop through vectors to compute t, I, S, D.
%using JPC's original Excel model probabilistic rule
for n=2:N
    t(n) = t(n-1) + dt;
    
    %Poisson probabalistic rule for transition from S to I
    if rand < dt*beta*S(n-1)*I(n-1)
        dS = -1;
        dI = 1;
    else
        dS = 0;
        dI = 0;
    end
    
    %Update I and S
    I(n) = I(n-1) + dI;
    S(n) = S(n-1) + dS;
    
    %Probabilistic rule for transition from I to D populations
    if rand < dt*alpha*I(n-1)
        dI = -1;
        dD = 1;
    else
        dI = 0;
        dD = 0;
    end
    
    %Update I and D (note I(n) is to be modified)
    I(n) = I(n) + dI;
    D(n) = D(n-1) + dD;
    
    %Make sure no population drops below zero
    S(n) = max( [0,S(n)] );
    I(n) = max( [0,I(n)] );
    D(n) = max( [0,D(n)] );
end

%%

%Stochastic model of Eyam SID model
function [t,I,S,D] = eyam_stochastic_model( dt, I0, S0, alpha, beta, tmax, lambda_multiplier )

%Initialize output vectors for t,I,S,D
t = 0 : dt : tmax;
N = length(t);
S = S0*ones(1,N);
I = I0*ones(1,N);
D = zeros(1,N);

%Loop through vectors to compute t, I, S, D.
%using a Poisson probabilistic rule for S,I,D changes during timestep dt
for n=2:N
    
    %Update time
    t(n) = t(n-1) + dt;
    
    %Randomly decide causality during time interval! i.e. Do we work out
    %the number of I to D changes first, then compute the S to I changes
    %based upon the updated I, or do we work out the S to I changes first,
    %update I and then work out the I to D changes?
    if rand > 0.5
        %Probabilistic rule for transition from I to D populations
        y = poisson_samples( dt*alpha*I(n-1), 1,lambda_multiplier);
        dD = y;
        dI = -y;
        
        %Update I and D
        I(n) = I(n-1) + dI;
        D(n) = D(n-1) + dD;
        
        %Poisson probabalistic rule for transition from S to I
        x = poisson_samples( dt*beta*S(n-1)*I(n), 1,lambda_multiplier );
        dS = -x;
        dI = x;
        
        %Update I and S
        I(n) = I(n) + dI;
        S(n) = S(n-1) + dS;
    else
        %Poisson probabalistic rule for transition from S to I
        x = poisson_samples( dt*beta*S(n-1)*I(n-1), 1,lambda_multiplier );
        dS = -x;
        dI = x;
        
        %Update S, I
        I(n) = I(n-1) + dI;
        S(n) = S(n-1) + dS;
        
        %Probabilistic rule for transition from I to D populations
        y = poisson_samples( dt*alpha*I(n), 1,lambda_multiplier);
        dD = y;
        dI = -y;
        
        %Update I and D
        I(n) = I(n) + dI;
        D(n) = D(n-1) + dD;
    end
    
    %Make sure no population drops below zero
    S(n) = max( [0,S(n)] );
    I(n) = max( [0,I(n)] );
    D(n) = max( [0,D(n)] );
end

%%

%Poisson distribution probabilities ( x is an array of integers >=0 )
function P = poisson(x,lambda)
P = (lambda.^x).*exp(-lambda)./factorial(x);

%%

%Function which generated samples from a Poisson distribution
function x = poisson_samples( lambda, N,lambda_multiplier )

%Determine Poisson probabilities for x = 0 .. ceil( 3*lambda );
if lambda > 0
    P = poisson( 0: ceil(lambda_multiplier*lambda),lambda );
else
    P = 0;
end

%Determine array of cumulative probabilities
CP = cumsum(P);
if CP(end)~=1
    CP = [CP,1];
end

%Generate N uniformly distributed random numbers in range [0,1]
z = rand(1,N);

%Define an array of cumulative probability, one element shifted.
%This is used to efficiently determine the 'bin' corresponding to z
CP0 = [0,CP(1:end-1)];

%For each number, determine the 'bin' number of a division of the interval
%[0,1] by the cumulative probabilities. If the first bin is zero, then
%these numbers should be distributed according to the probabililty
%distribution used to generate the cumulative probabilities.
x = zeros(1,N);
for k=1:length(CP)
    test = ( z >= CP0(k) ) & ( z <= CP(k) );
    x( test ) = k - 1;
end

%End of code